@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 #
 #--------------------------------------------------------------------------------
-#www2sms.py v1.0.5
+#www2sms.py v1.0.6
 #Copyright Bjoern Olausson
 #--------------------------------------------------------------------------------
 #This program is free software; you can redistribute it and/or modify
@@ -33,7 +33,7 @@ from www2sms_gui_dialog_progress import Ui_Form_Progress
 import www2sms_providers
 
 
-VERSION = "v1.0.5"
+VERSION = "v1.0.6"
 USER = pwd.getpwuid(os.getuid())[0]
 CONFIG_DIR = "/home/%s/MyDocs/www2sms" %(USER)
 
@@ -58,6 +58,14 @@ class MainWindow(QMainWindow, Ui_MainWindow):
 		self.ui.label_www2sms_version.setText(VERSION)
 		pixmap = QPixmap('/usr/share/pixmaps/www2sms.png')
 		self.ui.label_www2sms_logo.setPixmap(pixmap)
+		#The Account Form
+		class FormAccount(QDialog, Ui_FormAccount):
+			def __init__(self, win_parent = None):
+				#Init the base class
+				QDialog.__init__(self, win_parent)
+				self.ui = Ui_FormAccount()
+				self.ui.setupUi(self)
+		self.AccSet = FormAccount()
 		# Get the phone book
 		self.my_phone_book = self.get_contacts()
 		# Populate the contact list
@@ -65,28 +73,28 @@ class MainWindow(QMainWindow, Ui_MainWindow):
 			self.ui.comboBox_contact.addItem(contact[0])
 		# Populate the available Providers
 		for value in list_providers:
-			AccSet.ui.comboBox_SetupProvider.addItem(value)
+			self.AccSet.ui.comboBox_SetupProvider.addItem(value)
 		# Try to update the GUI to the last used Account, else skip this
 		try:
 			PROVIDER, ALIAS, LOGIN, PASSWORD = c.execute('SELECT provider, alias, login, password FROM login WHERE last=1').fetchall()[0]
 			# Populate all Acounts
 			self.ACCOUNTS = c.execute('SELECT alias FROM login').fetchall()
 			for a in self.ACCOUNTS:
-				AccSet.ui.comboBox_SetupAlias.addItem(a[0])
+				self.AccSet.ui.comboBox_SetupAlias.addItem(a[0])
 			# Update the GUI
 			# Updating the the Provider ComboBox
-			provider_index = AccSet.ui.comboBox_SetupProvider.findText(PROVIDER)
-			AccSet.ui.comboBox_SetupProvider.setCurrentIndex(provider_index)
+			provider_index = self.AccSet.ui.comboBox_SetupProvider.findText(PROVIDER)
+			self.AccSet.ui.comboBox_SetupProvider.setCurrentIndex(provider_index)
 			# Updating the the Alias ComboBox
-			alias_index = AccSet.ui.comboBox_SetupAlias.findText(ALIAS)
-			AccSet.ui.comboBox_SetupAlias.setCurrentIndex(alias_index)
+			alias_index = self.AccSet.ui.comboBox_SetupAlias.findText(ALIAS)
+			self.AccSet.ui.comboBox_SetupAlias.setCurrentIndex(alias_index)
 			# Updating the lineEdit Alias
-			AccSet.ui.lineEdit_SetupNewAlias.setText(ALIAS)
+			self.AccSet.ui.lineEdit_SetupNewAlias.setText(ALIAS)
 			# Updating the lineEdit Login
-			AccSet.ui.lineEdit_SetupLogin.setText(LOGIN)
+			self.AccSet.ui.lineEdit_SetupLogin.setText(LOGIN)
 			# Updating the lineEdit Pasword
 			#PASSWORD = base64.b64decode(PASSWORD)
-			AccSet.ui.lineEdit_SetupPassword.setText(PASSWORD)
+			self.AccSet.ui.lineEdit_SetupPassword.setText(PASSWORD)
 		except IndexError:
 			provider_index = -1
 			self.ACCOUNTS = False
@@ -95,10 +103,10 @@ class MainWindow(QMainWindow, Ui_MainWindow):
 		QtCore.QObject.connect(self.ui.pushButton_Send, QtCore.SIGNAL("clicked()"), self.on_send_clicked)
 		QtCore.QObject.connect(self.ui.comboBox_contact, QtCore.SIGNAL("activated(QString)"), self.comboBox_contact_itemActivated)
 		QtCore.QObject.connect(self.ui.comboBox_number, QtCore.SIGNAL("activated(QString)"), self.comboBox_number_itemActivated)
-		QtCore.QObject.connect(AccSet.ui.pushButton_SetupSave, QtCore.SIGNAL("clicked()"), self.on_SetupSave_clicked)
+		QtCore.QObject.connect(self.AccSet.ui.pushButton_SetupSave, QtCore.SIGNAL("clicked()"), self.on_SetupSave_clicked)
 		QtCore.QObject.connect(self.ui.plainTextEdit_SMS, QtCore.SIGNAL("textChanged()"), self.UpdateChars)
-		QtCore.QObject.connect(AccSet.ui.comboBox_SetupAlias, QtCore.SIGNAL("activated(QString)"), self.on_SetupAlias_activated)
-		QtCore.QObject.connect(AccSet.ui.pushButton_SetupDelete, QtCore.SIGNAL("clicked()"), self.on_SetupDelete_clicked)
+		QtCore.QObject.connect(self.AccSet.ui.comboBox_SetupAlias, QtCore.SIGNAL("activated(QString)"), self.on_SetupAlias_activated)
+		QtCore.QObject.connect(self.AccSet.ui.pushButton_SetupDelete, QtCore.SIGNAL("clicked()"), self.on_SetupDelete_clicked)
 		self.enableTabs(provider_index)
 		self.cpsms(provider_index)
 		QtCore.QObject.connect(self.ui.actionAdv_Provider_Settings, QtCore.SIGNAL("triggered(bool)"), self.ShowDialogAdv)
@@ -107,21 +115,21 @@ class MainWindow(QMainWindow, Ui_MainWindow):
     
 	def enableTabs(self, provider_index):
 		## Enable the SMS SEND Button if we have an account, otherwise not
-		if AccSet.ui.comboBox_SetupAlias.currentText():
+		if self.AccSet.ui.comboBox_SetupAlias.currentText():
 			self.ui.pushButton_Send.setEnabled(True)
 		else:
 		    	self.ui.pushButton_Send.setEnabled(False)
 		## Enable the "advanced" options if supported
 		if (provider_index >= 0) and providers[provider_index].hasAdvancedFeatures():
-			# TODO: replace "o2online_de" by the selected provider...
-			from www2sms_gui_widget_advanced_o2online_de import  Ui_FormAdvanced
-			class FormAdv(QDialog, Ui_FormAdvanced):
+			adv_module_name = "www2sms_gui_widget_advanced_%s" %( providers[provider_index].AdvModName() )	
+			AdvSetForm = __import__(adv_module_name)
+			class FormAdv(QDialog, AdvSetForm.Ui_FormAdvanced):
 				def __init__(self, win_parent = None):
 					#Init the base class
 					QDialog.__init__(self, win_parent)
-					self.ui = Ui_FormAdvanced()
+					self.ui = AdvSetForm.Ui_FormAdvanced()
 					self.ui.setupUi(self)
-			#The O2 Advanced Form
+			#The Advanced Form
 			self.AdvSet = FormAdv()
 			providers[provider_index].SetupGUI(self)
 			self.ui.actionAdv_Provider_Settings.setEnabled(True)
@@ -132,12 +140,12 @@ class MainWindow(QMainWindow, Ui_MainWindow):
 		self.AdvSet.show()
 
 	def ShowDialogAccSet(self):
-		AccSet.show()
+		self.AccSet.show()
 
 	# Function wich generates a nice sorted list for all contacts.
 	def get_contacts(self):
-		# Regular expression to find all phone numbers in the vCard
-		re_tel = re.compile('TEL;TYPE')
+		# Regular expression to find Cellular phone numbers in the vCard
+		re_tel = re.compile("TEL;.*TYPE=CELL.*")
 
 		# Not usefull right now...
 		name_attributes = [
@@ -202,6 +210,7 @@ class MainWindow(QMainWindow, Ui_MainWindow):
 			vcard = econtact.get_vcard_string()
 			for line in vcard.split('\n'):
 				if re_tel.search(line):
+					#print line
 					number = line.split(':')[1].strip()
 					contact_and_numbers.extend([number.decode('utf-8')])
 			# Now store the list of contact with all its numbers in in another list
@@ -250,7 +259,7 @@ class MainWindow(QMainWindow, Ui_MainWindow):
 		    	QMessageBox.information(self, 'www2sms', "Reached maximum allowd characters (%s)" %(self.max_c))
 		    	self.ui.pushButton_Send.setEnabled(False)
 		# Here we have to check if an Account exist, if not, we will not enable the send Button
-		if not self.ui.pushButton_Send.isEnabled() and SMS_CHARS <= self.max_c and AccSet.ui.comboBox_SetupAlias.currentText():
+		if not self.ui.pushButton_Send.isEnabled() and SMS_CHARS <= self.max_c and self.AccSet.ui.comboBox_SetupAlias.currentText():
 			self.ui.pushButton_Send.setEnabled(True)
 		# Might be better. Floor to Ceiling division: -(-x//y)
 		SMS_COUNT = int(math.ceil(SMS_CHARS / self.cps) + 1)
@@ -259,18 +268,18 @@ class MainWindow(QMainWindow, Ui_MainWindow):
 	# Method which updates the GUI to the Account selected in comboBox_SetupAlias
 	def on_SetupAlias_activated(self):
 		# Get the selected Account
-		ALIAS = unicode(AccSet.ui.comboBox_SetupAlias.currentText())
+		ALIAS = unicode(self.AccSet.ui.comboBox_SetupAlias.currentText())
 		# Read the settings for the selected Account fom DB
 		try:
 			PROVIDER, ALIAS, LOGIN, PASSWORD = c.execute('SELECT provider, alias, login, password FROM login WHERE alias=?', (ALIAS,)).fetchall()[0]
 		except IndexError:
 			pass
 		# Update the GUI elements
-		provider_index = AccSet.ui.comboBox_SetupProvider.findText(PROVIDER)
-		AccSet.ui.comboBox_SetupProvider.setCurrentIndex(provider_index)
-		AccSet.ui.lineEdit_SetupNewAlias.setText(ALIAS)
-		AccSet.ui.lineEdit_SetupLogin.setText(LOGIN)
-		AccSet.ui.lineEdit_SetupPassword.setText(PASSWORD)
+		provider_index = self.AccSet.ui.comboBox_SetupProvider.findText(PROVIDER)
+		self.AccSet.ui.comboBox_SetupProvider.setCurrentIndex(provider_index)
+		self.AccSet.ui.lineEdit_SetupNewAlias.setText(ALIAS)
+		self.AccSet.ui.lineEdit_SetupLogin.setText(LOGIN)
+		self.AccSet.ui.lineEdit_SetupPassword.setText(PASSWORD)
 		self.enableTabs(provider_index)
 		self.cpsms(provider_index)
 		# Remove the last account flag to set it on the new/updated profile
@@ -282,33 +291,33 @@ class MainWindow(QMainWindow, Ui_MainWindow):
 	# Well, this is obvious, isn't it? Okay, it deletes the current selected account.
 	def on_SetupDelete_clicked(self):
 		# Get the current selected Account
-		ALIAS = unicode(AccSet.ui.comboBox_SetupAlias.currentText())
+		ALIAS = unicode(self.AccSet.ui.comboBox_SetupAlias.currentText())
 		# Delete it from the DB
 		c.execute('DELETE FROM login WHERE alias=?', (ALIAS,))
 		conn.commit()
 		# Remove the Account from comboBox_SetupAlias
-		alias_index = AccSet.ui.comboBox_SetupAlias.findText(ALIAS)
-		AccSet.ui.comboBox_SetupAlias.removeItem(alias_index)
+		alias_index = self.AccSet.ui.comboBox_SetupAlias.findText(ALIAS)
+		self.AccSet.ui.comboBox_SetupAlias.removeItem(alias_index)
 		# Check if there is something left in the DB
 		ROWS = c.execute('SELECT alias FROM login').rowcount
 		# Update the GUI elements
 		if ROWS == -1:
-			AccSet.ui.lineEdit_SetupNewAlias.clear()
-			AccSet.ui.lineEdit_SetupLogin.clear()
-			AccSet.ui.lineEdit_SetupPassword.clear()
+			self.AccSet.ui.lineEdit_SetupNewAlias.clear()
+			self.AccSet.ui.lineEdit_SetupLogin.clear()
+			self.AccSet.ui.lineEdit_SetupPassword.clear()
 			self.enableTabs(-1)
 		else:
-			AccSet.on_SetupAlias_activated()
+			self.AccSet.on_SetupAlias_activated()
 		
-		QMessageBox.information(AccSet, 'www2sms', "Account %s deleted" %(ALIAS))
+		QMessageBox.information(self.AccSet, 'www2sms', "Account %s deleted" %(ALIAS))
 
 	# Method which is called, when an account is created/updated
 	def on_SetupSave_clicked(self):
 		# Gather Account information
-		PROVIDER = unicode(AccSet.ui.comboBox_SetupProvider.currentText())
-		ALIAS = unicode(AccSet.ui.lineEdit_SetupNewAlias.text())
-		LOGIN = unicode(AccSet.ui.lineEdit_SetupLogin.text())
-		PASSWORD = unicode(AccSet.ui.lineEdit_SetupPassword.text())
+		PROVIDER = unicode(self.AccSet.ui.comboBox_SetupProvider.currentText())
+		ALIAS = unicode(self.AccSet.ui.lineEdit_SetupNewAlias.text())
+		LOGIN = unicode(self.AccSet.ui.lineEdit_SetupLogin.text())
+		PASSWORD = unicode(self.AccSet.ui.lineEdit_SetupPassword.text())
 		# Remove the last account flag to set it on the new/updated profile
 		c.execute('UPDATE login SET last=0 WHERE last=1')
 		# Check if Account exist, if it exists, we update the content in the DB on save, else we add it
@@ -316,19 +325,19 @@ class MainWindow(QMainWindow, Ui_MainWindow):
 		if len(EXIST) != 0:
 			c.execute('UPDATE login SET provider=?, login=?, password=?, last=1 WHERE alias=?', (PROVIDER, LOGIN, PASSWORD, ALIAS))
 		else:
-			c.execute('INSERT INTO login VALUES (NULL, ?, ?, ?, ?, 1)', (PROVIDER, ALIAS, LOGIN, PASSWORD))
+			c.execute('INSERT INTO login (id, provider, alias, login, password, last)VALUES (NULL, ?, ?, ?, ?, 1)', (PROVIDER, ALIAS, LOGIN, PASSWORD))
 			# Add the new Account to comboBox_SetupAlias
-		    	AccSet.ui.comboBox_SetupAlias.addItem(ALIAS)
+		    	self.AccSet.ui.comboBox_SetupAlias.addItem(ALIAS)
 		conn.commit()
 		# Let the user know that we have saved the the account/changes
-		QMessageBox.information(AccSet, 'www2sms', "saved")
+		QMessageBox.information(self.AccSet, 'www2sms', "saved")
 		# Update the GUI
-		alias_index = AccSet.ui.comboBox_SetupAlias.findText(ALIAS)
-		AccSet.ui.comboBox_SetupAlias.setCurrentIndex(alias_index)
-		provider_index = AccSet.ui.comboBox_SetupProvider.findText(PROVIDER)
-		AccSet.ui.comboBox_SetupProvider.setCurrentIndex(provider_index)
-		AccSet.ui.lineEdit_SetupLogin.setText(LOGIN)
-		AccSet.ui.lineEdit_SetupPassword.setText(PASSWORD)
+		alias_index = self.AccSet.ui.comboBox_SetupAlias.findText(ALIAS)
+		self.AccSet.ui.comboBox_SetupAlias.setCurrentIndex(alias_index)
+		provider_index = self.AccSet.ui.comboBox_SetupProvider.findText(PROVIDER)
+		self.AccSet.ui.comboBox_SetupProvider.setCurrentIndex(provider_index)
+		self.AccSet.ui.lineEdit_SetupLogin.setText(LOGIN)
+		self.AccSet.ui.lineEdit_SetupPassword.setText(PASSWORD)
 		self.enableTabs(provider_index)
 		self.cpsms(provider_index)
 
@@ -344,8 +353,8 @@ class MainWindow(QMainWindow, Ui_MainWindow):
 			QMessageBox.warning(self, 'www2sms', 'Please connect to the Internet bofore sending the SMS')
 			return
 	    	# Read the provider to use
-		PROVIDER = unicode(AccSet.ui.comboBox_SetupProvider.currentText())
-		provider_index = AccSet.ui.comboBox_SetupProvider.findText(PROVIDER)
+		PROVIDER = unicode(self.AccSet.ui.comboBox_SetupProvider.currentText())
+		provider_index = self.AccSet.ui.comboBox_SetupProvider.findText(PROVIDER)
 		provider = providers[provider_index]
 	    	#Check if a receiver number was provided
 		# Read the number to which we send the SMS
@@ -381,9 +390,11 @@ class MainWindow(QMainWindow, Ui_MainWindow):
 		if reply == QMessageBox.Yes:
 			self.Stat = FormPro()
 			# Read the login
-			LOGIN = unicode(AccSet.ui.lineEdit_SetupLogin.text())
+			LOGIN = unicode(self.AccSet.ui.lineEdit_SetupLogin.text())
 			# Read the password
-			PASSWORD = unicode(AccSet.ui.lineEdit_SetupPassword.text())
+			PASSWORD = unicode(self.AccSet.ui.lineEdit_SetupPassword.text())
+			# Read the ALIAS
+			ALIAS = unicode(self.AccSet.ui.lineEdit_SetupNewAlias.text())
 			# send the SMS
 			STATUS, MSG = provider.sendSMS(self, app, NUMBER, self.SMSTEXT, LOGIN, PASSWORD)
 			self.Stat.close()
@@ -411,13 +422,6 @@ class MainWindow(QMainWindow, Ui_MainWindow):
 def DialogStatusMessage(self, statusmessage):
 	QMessageBox.information(self, 'www2sms', statusmessage)
 
-class FormAccount(QDialog, Ui_FormAccount):
-	def __init__(self, win_parent = None):
-		#Init the base class
-		QDialog.__init__(self, win_parent)
-		self.ui = Ui_FormAccount()
-		self.ui.setupUi(self)
-
 class FormPro(QDialog, Ui_Form_Progress):
 	def __init__(self, win_parent = None):
 		#Init the base class
@@ -426,6 +430,10 @@ class FormPro(QDialog, Ui_Form_Progress):
 		self.ui.setupUi(self)
 
 def mk_db():
+    #Table login
+    #id|provider|alias|login|password|advanced|last
+    #01|aaaaa   |bbbb |cccc |dddd    |eeeee   |1
+    #...
     try:
 	os.makedirs(CONFIG_DIR, 0755)
     except OSError, e:
@@ -440,11 +448,17 @@ def mk_db():
 	c = conn.cursor()
 	# Create Database
 	try:
-	    c.execute('''create table login (id INTEGER PRIMARY KEY, provider TEXT, alias TEXT UNIQUE, login TEXT, password TEXT, last INTEGER)''')
+	    c.execute('''create table login (id INTEGER PRIMARY KEY, provider TEXT, alias TEXT UNIQUE, login TEXT, password TEXT, advanced TEXT, last INTEGER)''')
 	    #c.execute('''create table o2online (id INTEGER PRIMARY KEY, provider TEXT, alias TEXT UNIQUE)''')
 	    conn.commit()
 	except sqlite3.OperationalError:
 	    pass
+	try:
+	    c.execute('''alter table login add column advanced TEXT''')
+	except sqlite3.OperationalError:
+	    pass
+	#c.execute('SELECT * FROM login')
+	#print c.fetchall()
 	c.close()
 	conn.close()
 
@@ -536,8 +550,6 @@ if __name__ == "__main__":
 	app = QApplication(sys.argv)
 	#The Status Form
 	#Stat = FormPro()
-	#The Account Form
-	AccSet = FormAccount()
 	#The Main window
 	ui = MainWindow()
 	ui.show()
