@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-import sys, re, mechanize, base64, urllib2, time, calendar
+import sys, re, mechanize, base64, urllib2, time, calendar, urllib, sqlite3
 from PyQt4 import QtCore, QtGui
 from PyQt4.QtGui import QApplication, QDialog, QImage, QPixmap
 from www2sms_gui_dialog_captcha import Ui_DialogCaptcha
@@ -51,11 +51,14 @@ class provider():
 		return False, "SMS sending not implemented"
 
 # o2online.de 2.2
+# o2online (DE) Provider written by Bjoern Olausson <maemo@olausson.de>
 class provider_o2online_de(provider):
 	def getName(self):
-		return 'o2online (Deutschland)'
+		return 'o2online (Germany)'
 	def hasAdvancedFeatures(self):
 		return True
+	def AdvModName(self):
+		return 'o2online_de'
 	def SetupGUI(self, MainWindow):
 		CT = time.localtime(time.time())
 		# Set the year for scheduling SMS. From now to the next three years
@@ -331,7 +334,7 @@ class provider_o2online_de(provider):
 				return error(self, MainWindow, e, 'Failed to open the SMS-Center')
 
 			# Find how much FREE SMS we have left.
-			FREESMS = re_free.findall(response)[0].split()[1]
+			FREESMS = int(re_free.findall(response)[0].split()[1])
 
 			# Select the SMS-Form
 			try:
@@ -379,16 +382,18 @@ class provider_o2online_de(provider):
 				br.submit()
 				response = br.response().read()
 				if re_suc.search(str(response)):
-					return error(self, MainWindow, True, 'Provider responded: Ihre SMS wurde erfolgreich versendet')
+					SMS_LEFT = FREESMS - 1
+					return error(self, MainWindow, True, 'Provider responded: Ihre SMS wurde erfolgreich versendet\nVerbleibende SMS: %s' %(SMS_LEFT))
 				else:
-					return error(self, MainWindow, False, 'Provider responded: Ihre SMS wurde nicht versendet')
+					return error(self, MainWindow, False, 'Provider responded: Ihre SMS wurde nicht versendet\nVerbleibende SMS: %s' %(FREESMS))
 			except Exception, e:
 					return error(self, MainWindow, e, 'Submission faild')
 
 # vodafone.nl 1.0
+# Vodafone (NL) Provider written by Shane Kerr <shane@time-travellers.org>
 class provider_vodafone_nl(provider):
 	def getName(self):
-		return 'Vodefone Nederland'
+		return 'Vodefone (Nederland)'
 	def hasAdvancedFeatures(self):
 		return False
 	def chars_per_sms(self):
@@ -474,3 +479,260 @@ class provider_vodafone_nl(provider):
 		else:
 			return False, 'Message not sent'
         	MainWindow.Stat.close()
+
+# Exetel Australia
+# API Docs: http://www.exetel.com.au/Exetel_SMS_API_documentation.pdf
+# Exetel Provider written by Janeene Beeforth <n900@dawnmist.net>
+class provider_exetel_au(provider):
+	def getName(self):
+		return 'Exetel (Australia)'
+	def hasAdvancedFeatures(self):
+		return True
+	def AdvModName(self):
+		return 'exetel_au'
+	def chars_per_sms(self):
+		return 160, 608
+	def num_regexp(self, num):
+	    	# Get rid of everything except 
+		num = re.sub(r'[^(0-9)|\+]', '', num)
+		# handle a couple of number formats
+		if re.search(r'^\+\d{11}$', num):	# ITU format
+			return True, num, 'Valid Number'
+		elif re.search(r'^\d{11}$', num):	# ITU format with no +
+			return True, num, 'Valid Number'
+		elif re.search(r'^0\d{9}$', num):	# local dialing format
+			return True, num, 'Valid Number'
+		else:
+			return False, num, 'Unknown number format '.number
+
+	def SetupGUI(self, MainWindow):
+		conn = sqlite3.connect('www2sms.db')
+		c = conn.cursor()
+		ALIAS = unicode(MainWindow.AccSet.ui.lineEdit_SetupNewAlias.text())
+		ADVANCED = ""
+		try:
+			ADVANCED = str(c.execute('select advanced FROM login WHERE alias=?', (ALIAS,)).fetchall()[0][0])
+		except sqlite3.OperationalError:
+			pass
+		if ADVANCED != "None":
+			MainWindow.AdvSet.ui.lineEdit_AdvancedText.setText(ADVANCED)
+		c.close()
+		conn.close()
+		CT = time.localtime(time.time())
+		# Set the year for scheduling SMS. From now to the next three years
+		YEAR = CT[0]
+		for YEARS in range(YEAR,YEAR + 3, 1):
+			MainWindow.AdvSet.ui.comboBox_AdvancedScheduleStartYear.addItem(str(YEARS))
+		# Method which is called to get the proper number of days for the schedule start date
+		def on_StartDate_changed(self):
+			cal = calendar.Calendar()
+			sYEAR = int(MainWindow.AdvSet.ui.comboBox_AdvancedScheduleStartYear.currentText())
+			sMONTH = int(MainWindow.AdvSet.ui.comboBox_AdvancedScheduleStartMonth.currentText())
+			MainWindow.AdvSet.ui.comboBox_AdvancedScheduleStartDay.clear()
+			for DAY in cal.itermonthdays(sYEAR, sMONTH):
+				if DAY != 00:
+					DAY = '%02d' %(DAY)
+					MainWindow.AdvSet.ui.comboBox_AdvancedScheduleStartDay.addItem(str(DAY))
+
+		def on_AdvSetSave_clicked():
+			ALIAS = unicode(MainWindow.AccSet.ui.lineEdit_SetupNewAlias.text())
+			conn = sqlite3.connect('www2sms.db')
+			c = conn.cursor()
+			DB_SENDER = str(c.execute('select advanced FROM login WHERE alias=?', (ALIAS,)).fetchall()[0][0])
+			SENDER = str(MainWindow.AdvSet.ui.lineEdit_AdvancedText.text())
+			if SENDER != "":
+			    if DB_SENDER == "None" or SENDER != DB_SENDER :
+				c.execute('UPDATE login SET advanced=? WHERE alias=?', (SENDER, ALIAS))
+				conn.commit()
+			c.close()
+			conn.close()
+			# Let the user know that we have saved the the account/changes
+			QtGui.QMessageBox.information(MainWindow.AdvSet, 'www2sms', "saved")
+
+		QtCore.QObject.connect(MainWindow.AdvSet.ui.comboBox_AdvancedScheduleStartMonth, QtCore.SIGNAL("activated(QString)"), on_StartDate_changed)
+		QtCore.QObject.connect(MainWindow.AdvSet.ui.comboBox_AdvancedScheduleStartYear, QtCore.SIGNAL("activated(QString)"), on_StartDate_changed)
+		QtCore.QObject.connect(MainWindow.AdvSet.ui.pushButton_Save, QtCore.SIGNAL("clicked()"), on_AdvSetSave_clicked)
+
+
+	def get_advanced_setting(self, MainWindow):
+		# Read Sender
+		SENDER = MainWindow.AdvSet.ui.lineEdit_AdvancedText.text()
+		# Read Scheduling settings if activated
+		SCHEDULE = []
+		if MainWindow.AdvSet.ui.checkBox_AdvancedSchedule.isChecked():
+			# SCHEDULE Layout: ([TrueFalse], [YEARstart], [MONTHstart], [DAYstart], [HOURstart], [MINUTEstart])
+			SCHEDULE.append([ MainWindow.AdvSet.ui.checkBox_AdvancedSchedule.isChecked() ])
+			SCHEDULE.append([ str(MainWindow.AdvSet.ui.comboBox_AdvancedScheduleStartYear.currentText())])
+			SCHEDULE.append([ str(MainWindow.AdvSet.ui.comboBox_AdvancedScheduleStartMonth.currentText()) ])
+			SCHEDULE.append([ str(MainWindow.AdvSet.ui.comboBox_AdvancedScheduleStartDay.currentText()) ])
+			SCHEDULE.append([ str(MainWindow.AdvSet.ui.comboBox_AdvancedScheduleStartHour.currentText()) ])
+			SCHEDULE.append([ str(MainWindow.AdvSet.ui.comboBox_AdvancedScheduleStartMinute.currentText()) ])
+		else:
+			SCHEDULE = False
+		# Don't need Special
+		FLASH = False
+
+		return SCHEDULE, SENDER, FLASH
+
+	def sendSMS(self, MainWindow, app, NUMBER, SMSTEXT, LOGIN, PASSWORD):
+		LOGINURL = 'https://www.exetel.com.au/'
+                # XXX: GUI manipulation
+		SCHEDULE, SENDER, FLASH = self.get_advanced_setting(MainWindow)
+		# Refuse to send without sender number
+		if (str(SENDER) == ""):
+			return False, 'Not Sent: Exetel requires a Sender number. Please set this in Adv Provider settings in the "Sender" field.'
+		
+                MainWindow.Stat.show()
+		MainWindow.Stat.ui.progressBar.setMinimum(0)
+		MainWindow.Stat.ui.progressBar.setMaximum(3)
+
+		# set up our browser
+		UpdatePbar(self, MainWindow, app, "1", "Initialising the browser")
+		try:
+			br = mechanize.Browser()
+		except Exception, e:
+			return False, "Failed to open browser"
+
+		# strip the '+' if it's still here from the number to send to
+		if re.search(r'^\+\d{11}$', NUMBER):	# ITU format - strip the +
+			num = re.sub(r'[^(0-9)]', '', NUMBER)
+		else:
+			num = NUMBER
+
+		# Scheduling
+		if SCHEDULE :
+			address = 'https://smsgw.exetel.com.au/sendsms/api_sms_schedule.php?username=%s&sender=%s&password=%s&mobilenumber=%s&message=%s&messagetype=Text&requesttype=ADD&scheduledday=%s&scheduledmonth=%s&scheduledyear=%s&scheduledhour=%s&scheduledminute=%s' %(LOGIN, SENDER, PASSWORD, num, urllib.quote_plus(SMSTEXT), SCHEDULE[3][0], SCHEDULE[2][0], SCHEDULE[1][0], SCHEDULE[4][0], SCHEDULE[5][0])
+		
+		else:
+			address = 'https://smsgw.exetel.com.au/sendsms/api_sms.php?username=%s&sender=%s&password=%s&mobilenumber=%s&message=%s&messagetype=Text' %(LOGIN, SENDER, PASSWORD, num, urllib.quote_plus(SMSTEXT))
+		try:
+			# Uncomment below line for debugging...
+			#print("Sending to '" + address + "'")
+			UpdatePbar(self, MainWindow, app, "2", 'Sending Message')
+			br.open(address)
+		except:
+			return False, 'Error sending message: Failed to open ' + address
+		
+		UpdatePbar(self, MainWindow, app, "3", "Checking Reply")
+		result = br.response().read().split('|')
+		if re.search(r'1', result[0]):
+			return True, 'Sent. Provider responded: ' + result[4].replace('<br>', '')
+		elif re.search(r'2', result[0]):
+			return False, 'Not Sent. Provider responded:  ' + result[4]
+		else:
+			return False, 'Failed. Provider responded: ' + result[4]
+
+        	MainWindow.Stat.close()
+
+
+# MutluSMS 1.0 (API v0.2)
+# API Docs: http://www.mutlusms.com/yazilimgelistiriciler.html | http://www.mutlusms.com/js/MutluSmsHttpApi.pdf
+# MutluSMS (TR) Provider written by Bjoern Olausson <maemo@olausson.de>
+class provider_MutluSMS_tu(provider):
+	def getName(self):
+		return 'MutluSMS (Turkey)'
+	def hasAdvancedFeatures(self):
+		return True
+	def AdvModName(self):
+		return 'mutlusms_tu'
+	def chars_per_sms(self):
+		return 160, 320
+	def num_regexp(self, num):
+		# Remove everything except numbers and "+"
+		num = re.sub(r'[^(0-9)|\+]', '', num)
+		#Since we can send SMS anywhere it does not make sense to check for vlidity
+		return True, num, 'Valid Number'
+		#TODO Check if the number matches either +905321234567 , 905321234567 , 05321234567
+		#TODO Handy Vorwahl Türkei (Turkey): 0090542851-8, 0090542871-4, 0090542882-6, 0090542889, 009054, 009053383-7, 009050, 0090551-6, 0090559, 00905, 009053
+		#if re.match('^(\+905[0-9]+)|^(905[0-9]+)|^(05[0-9]+)', num):
+			#return True, num, 'Valid Number'
+		#else:
+			#return False, num, 'Invalid formated number. Valid examples: +905321234567 , 905321234567 , 05321234567'
+
+	def SetupGUI(self, MainWindow):
+		conn = sqlite3.connect('www2sms.db')
+		c = conn.cursor()
+		ALIAS = unicode(MainWindow.AccSet.ui.lineEdit_SetupNewAlias.text())
+		ADVANCED = ""
+		try:
+			ADVANCED = str(c.execute('select advanced FROM login WHERE alias=?', (ALIAS,)).fetchall()[0][0])
+		except sqlite3.OperationalError:
+			pass
+		if ADVANCED != "None":
+			MainWindow.AdvSet.ui.lineEdit_AdvancedText.setText(ADVANCED)
+		c.close()
+		conn.close()
+
+		def on_AdvSetSave_clicked():
+			ALIAS = unicode(MainWindow.AccSet.ui.lineEdit_SetupNewAlias.text())
+			conn = sqlite3.connect('www2sms.db')
+			c = conn.cursor()
+			DB_SENDER = str(c.execute('select advanced FROM login WHERE alias=?', (ALIAS,)).fetchall()[0][0])
+			SENDER = str(MainWindow.AdvSet.ui.lineEdit_AdvancedText.text())
+			if SENDER != "":
+			    if DB_SENDER == "None" or SENDER != DB_SENDER :
+				c.execute('UPDATE login SET advanced=? WHERE alias=?', (SENDER, ALIAS))
+				conn.commit()
+			c.close()
+			conn.close()
+			# Let the user know that we have saved the the account/changes
+			QtGui.QMessageBox.information(MainWindow.AdvSet, 'www2sms', "saved")
+
+		QtCore.QObject.connect(MainWindow.AdvSet.ui.pushButton_Save, QtCore.SIGNAL("clicked()"), on_AdvSetSave_clicked)
+
+	def get_advanced_setting(self, MainWindow):
+		# Read Sender
+		SENDER = MainWindow.AdvSet.ui.lineEdit_AdvancedText.text()
+		# Flash SMS?
+		FLASH = MainWindow.AdvSet.ui.checkBox_AdvancedFlashSMS.isChecked()
+		if FLASH:
+		    FLASH = 1
+		else:
+		    FLASH = 0
+		return SENDER, FLASH
+
+	def sendSMS(self, MainWindow, app, NUMBER, SMSTEXT, LOGIN, PASSWORD):
+		SENDER, FLASH = self.get_advanced_setting(MainWindow)
+		if SENDER == "":
+			return False, 'A sender is required in the advanced Setup'
+		MainWindow.Stat.show()
+		MainWindow.Stat.ui.progressBar.setMinimum(0)
+		MainWindow.Stat.ui.progressBar.setMaximum(2)
+		ERROR_MSG = {
+			    "11":"Message is too long",
+			    "12": "Message has unusable characters: ^ { } [ ] ~ \ |",
+			    "13": "Wrong phone number(not sender, the one which gonna get sms)",
+			    "14": "Wrong sender name",
+			    "15": "Unsupported sender number",
+			    "16": "Unsupported sender name",
+			    "17": "Flash sms cant have special characters",
+			    "18": "Wrong parameter",
+			    "19": "You cant send more than 100 person once.",
+			    "101": "Wrong authority info",
+			    "102": "Wrong username or password",
+			    "103": "Not enough credits",
+			    "301": "This registry number couldnt found!",
+			    "999": "Error about mutlusms (Contact to MutluSMS)"
+			    }
+		#API URL
+		url = "http://www.mutlusms.com/api/gonder.do"
+		#Remove invalid chars from message
+		mesaj = re.sub(r'\^|\{|\}|\[|\]|\~|\\|\|', '', SMSTEXT)
+		#Assemble REQUEST
+		urlSon = url + "?id=%s&sifre=%s&gonderen=%s&mesaj=%s&telefon=%s&flash=%s" %(LOGIN,PASSWORD,SENDER,mesaj,NUMBER,FLASH)
+		UpdatePbar(self, MainWindow, app, "1", "Sending SMS")
+		print "VARS:", SENDER, FLASH, LOGIN, PASSWORD, mesaj, NUMBER
+		print "URL: ", urlSon
+		try:
+			result = urllib2.urlopen(urlSon).read()
+		except URLError, e:
+			return False, 'Error connecting to service:%s\n\n %s' %(e.reason, result)
+		except HTTPError, e:
+			return False, 'Error %s returnd from service\n\n %s' %(e.code, result)
+		UpdatePbar(self, MainWindow, app, "2", "Checking reply")
+		re_suc = re.compile('OK')
+		if re_suc.search(result):
+		    return True, 'Sent. Message ID: %s' %(result.strip("OK:"))
+		else:
+		    return False, 'Failed. Provider responded: %s' %(ERROR_MSG[result.strip("HATA:")])
+		MainWindow.Stat.close()
